program define iso_terawas
version 10 /*stata version*/
/*program version 3.0*/


display
display
display
display as text". ISOTOOLS by F. Bea (2007); v. 3.0 (20 Jun 2008)"
display
display as text". Tera-Wasserburg Concordia Plot and Age Calculations"
display
display

/*checking if neccessary variables are present*/
	capture confirm string variable id
	if _rc~=0 {
		display as error". YOU DO NOT HAVE A STRING VARIABLE NAMED -id-"
		exit
	}
	capture confirm numeric variable pb7_pb6
	if _rc~=0 {
		display as error". YOU DO NOT HAVE A NUMERIC VARIABLE NAMED pb7_pb6"
		exit
	}
	capture confirm numeric variable pb6_u8
	if _rc~=0 {
		display as error". YOU DO NOT HAVE A NUMERIC VARIABLE NAMED pb6_u8"
		exit
	}
	capture confirm numeric variable u_ppm
	if _rc~=0 {
		display as error". YOU DO NOT HAVE A NUMERIC VARIABLE NAMED u_ppm. I cann't correct for U excess"
		local u_correct=0
	}
	else{
		local u_correct=1
		capture confirm numeric variable flag_4cor
		if _rc~=0 {
		gen flag_4cor=0
		}
	}	
	capture confirm numeric variable flag_uexcesscor
	if _rc~=0 {
	gen flag_uexcesscor=0
	}
/*end of checking if neccessary variables are present*/

local ndata=_N

/* U excess correction, according to Ian Williams*/ 
quietly{
 	if `u_correct'==1{
 		if flag_uexcesscor~=1{
 			replace flag_uexcesscor=0 if flag_uexcesscor~=1
 			replace pb6_u8=pb6_u8*(1-0.02*(u_ppm-2500)/1000) if u_ppm>2500 & flag_uexcesscor==0
 			replace flag_uexcesscor=1 if u_ppm>2500
 		}/* del if flag_uexcesscor*/
 	}/*del if u_correct*/ 	
 }/*del quietly*/	
/*end U excess correction*/

/*204 correction*/
local correction_204 "n"
capture confirm numeric variable pb4_pb6
	if _rc==0 {
		capture confirm string `correction_204'
		if _rc!=0 {
             display as result". Shall I correct common lead according to measured 204/206? (y/n) " _cont
             display _request(_correction_204)
             }
             quietly{
             if  "`correction_204'"=="y" {
    
             		replace flag_4cor=1
             		capture drop pb7_pb6_4cor
             		capture drop pb8_pb6_4cor
             		capture drop pb6_u8_4cor
             		capture drop pb8_th_4cor
             		gen pb7_pb6_4cor=.
             		gen pb8_pb6_4cor=.
             		gen pb6_u8_4cor=.
             		gen pb8_th_4cor=.
             		global age 0
					global i 0
					global pb208 0
					global pb204 0

				while $i<[_N] {/*while 1*/
						global i=$i +1
						global pb207=pb7_pb6[$i]
						global pb208=pb8_pb6[$i]
						global pb204=pb4_pb6[$i]
						global t_pbpb=0	
						t_pbpb $pb207					
						local control=$t_pbpb*2
						while int($t_pbpb*10) !=int(`control'*10) {/*while2*/
							local control=$t_pbpb 
							sk	$pb207 $pb208 $pb204	
							t_pbpb $pb207_4cor
						}/*del while 2*/
				
					replace pb7_pb6_4cor= $pb207_4cor in $i
					replace pb8_pb6_4cor= $pb208_4cor in $i
					replace pb6_u8_4cor=$pb6_u8_4cor in $i
					replace pb8_th_4cor=$pb8_th_4cor in $i
				 }/*del while 1*/
			 }/* if de la correction_204*/
			 else {
			 		capture drop flag_4cor
			 		gen flag_4cor=0
			 		}
			 	}/*del quietly*/	
 		}/*del primer if*/
 /*end 204 correction*/
 
 
/* calculos*/
	local graph_title "TW plot. Common-lead uncorrected"
	if  "`correction_204'"=="y" {
		quietly{
			capture drop pb7_pb6_uncor pb6_u8_uncor pb8_th_uncor
			gen pb7_pb6_uncor=pb7_pb6
			gen pb6_u8_uncor=pb6_u8
			gen pb8_th_uncor=pb8_th
			replace pb7_pb6=pb7_pb6_4cor
			replace pb6_u8=pb6_u8_4cor
			replace pb8_th=pb8_th_4cor
			local graph_title "TW plot. 204-corrected"
			}/*del quietly*/
		}

	quietly{
		drop if id=="to_plot"
		}
	capture drop caso t_plot label
	capture drop u8_pb6 u5_pb7 pb7_u5 th_pb8
	capture drop t_u8_pb6 t_pb7_pb6 t_u5_pb7 t_th_pb8
	capture drop t_u8_pb6_4cor t_pb7_pb6_4cor t_u5_pb7_4cor t_th_pb8_4cor
	capture drop hi_pb7_pb6 lo_pb7_pb6 hi_u8_pb6 lo_u8_pb6
	capture drop err_t_u8_pb6 err_t_u5_pb7 err_t_th_pb8 err_t_pb7_pb6 

	quietly{/*calcula el resto de las relaciones isotpicas*/
		gen u8_pb6=1/pb6_u8
		gen u5_pb7=(0.72/99.2743)*u8_pb6/pb7_pb6
		gen pb7_u5=1/u5_pb7
		gen th_pb8=1/pb8_th
		}

	quietly {/*los mximos y minimos para las barras de error*/
		capture drop hi_pb7_pb6 hi_pb7_pb6 lo_pb7_pb6 hi_u8_pb6 lo_u8_pb6
		gen hi_pb7_pb6=pb7_pb6*(1+err_pb7_pb6/100)
		gen lo_pb7_pb6=pb7_pb6*(1-err_pb7_pb6/100)
		gen hi_u8_pb6=u8_pb6*(1+err_pb6_u8/100)
		gen lo_u8_pb6=u8_pb6*(1-err_pb6_u8/100)
		}

	quietly {/*comienza el calculo de edades*/
			gen t_u8_pb6=(10000/1.55125)*log(1+1/u8_pb6)
			gen err_t_u8_pb6=t_u8_pb6-(10000/1.55125)*log(1+1/(u8_pb6*(1+err_pb6_u8/100)))
			gen t_u5_pb7=(10000/9.8485)*log(1+1/u5_pb7)
			gen err_t_u5_pb7=t_u5_pb7-(10000/9.8485)*log(1+1/(u5_pb7*(1+err_pb6_u8/100)))
			gen t_th_pb8=(100000/4.9475)*log(1+1/th_pb8)
			gen err_t_th_pb8=t_th_pb8-(100000/4.9475)*log(1+1/(th_pb8*(1+err_pb8_th/100)))

			gen float t_pb7_pb6=0 /*las edades pb7_pb6*/
			forvalues i=1(1)`ndata' {
				global pbpb=pb7_pb6[`i']
				t_pbpb $pbpb
				replace t_pb7_pb6 = $t_pbpb in `i'
				}/*del forvalues*/
			gen float err_t_pb7_pb6=0 /*los errores en las edades pb7_pb6*/
			forvalues i=1(1)`ndata' {
				global pbpb=lo_pb7_pb6[`i']
				t_pbpb $pbpb
				replace err_t_pb7_pb6 = t_pb7_pb6[`i']-$t_pbpb in `i'
			}/*del forvalues*/

			/*207 correction*/
		if  "`correction_204'"~="y" {
				capture drop t_u8_pb6_7cor err_t_u8_pb6_7cor
				gen float t_u8_pb6_7cor=0 	/*las edades corregidas de Pb comun con 207*/
				forvalues i=1(1)`ndata' {
					global t_upb=t_u8_pb6[`i']
					global pbpb=pb7_pb6[`i']
				local t1=`t2'+10
						cor207 $t_upb $pbpb
					replace t_u8_pb6_7cor= $t_upbcor in `i'
	  				 }/*del forvalues*/

	  			gen float err_t_u8_pb6_7cor=0 /*los errores de las edades corregidas de Pb comun con 207*/
	 	 		forvalues i=1(1)`ndata' {/*207 correction when no 204 correction has been made*/
	  				global pbpb=pb7_pb6[`i']
	  				global alfa=lo_u8_pb6[`i']/(0.835722-lo_pb7_pb6[`i'])
	  				local hi_upb=hi_u8_pb6[`i']+$alfa*($pbpb-lo_pb7_pb6[`i'])
					global t_upb=(10000/1.55125)*log(1+1/`hi_upb')
					local t1=`t2'+10
					cor207 $t_upb $pbpb
					replace err_t_u8_pb6_7cor= t_u8_pb6_7cor[`i']-$t_upbcor in `i'
					}/*del forvalues*/
				}

		if  "`correction_204'"=="y" { /*207 correction when 204 correction has been also made*/
				capture drop t_u8_pb6_7cor err_t_u8_pb6_7cor
				gen float t_u8_pb6_7cor=0 	/*las edades corregidas de Pb comun con 207*/
				forvalues i=1(1)`ndata' {
					global t_upb=(10000/1.55125)*log(1+pb6_u8_uncor[`i'])
					global pbpb=pb7_pb6_uncor[`i']
					local t1=`t2'+10
					cor207 $t_upb $pbpb
					replace t_u8_pb6_7cor= $t_upbcor in `i'
	  			 	}/*del forvalues*/

	  			gen float err_t_u8_pb6_7cor=0 /*los errores de las edades corregidas de Pb comun con 207*/
	  			quietly {/*los mximos y minimos para las barras de error*/  			
					capture drop hi_pb7_pb6_uncor hi_pb7_pb6_uncor lo_pb7_pb6_uncor hi_u8_pb6_uncor lo_u8_pb6_uncor
					gen hi_pb7_pb6_uncor=pb7_pb6_uncor*(1+err_pb7_pb6/100)
					gen lo_pb7_pb6_uncor=pb7_pb6_uncor*(1-err_pb7_pb6/100)
					gen hi_u8_pb6_uncor=(1/pb6_u8_uncor)*(1+err_pb6_u8/100)
					gen lo_u8_pb6_uncor=(1/pb6_u8_uncor)*(1-err_pb6_u8/100)
						}
						
	 	 		forvalues i=1(1)`ndata' {
	  				global pbpb=pb7_pb6_uncor[`i']
	  				global alfa=lo_u8_pb6_uncor[`i']/(0.835722-lo_pb7_pb6_uncor[`i'])
	  				local hi_upb=hi_u8_pb6_uncor[`i']+$alfa*($pbpb-lo_pb7_pb6_uncor[`i'])
					global t_upb=(10000/1.55125)*log(1+1/`hi_upb')
					local t1=`t2'+10
					cor207 $t_upb $pbpb
					replace err_t_u8_pb6_7cor= t_u8_pb6_7cor[`i']-$t_upbcor in `i'
					}/*del forvalues*/
					capture drop hi_pb7_pb6_uncor hi_pb7_pb6_uncor lo_pb7_pb6_uncor hi_u8_pb6_uncor lo_u8_pb6_uncor
				}/*del if 204_corr*/
	}/*end quietly de las edades*/
	
	/*aqui hago las lista*/

		format %9.1f t_u8_pb6_7cor t_u8_pb6 t_u5_pb7 t_pb7_pb6 t_th_pb8 err_t_u8_pb6 err_t_u8_pb6_7cor err_t_u5_pb7 err_t_pb7_pb6 err_t_th_pb8
			if  "`correction_204'"=="y" display as text"204-corrected ages"
			if  "`correction_204'"~="y" display as text"Common-lead uncorrected ages"
		display as text "        id      207-cor(1s)  206Pb/238U(1s)  207Pb/235U(1s)  207Pb/206Pb(1s)  208Pb/232Th(1s)"   
 		list id t_u8_pb6_7cor err_t_u8_pb6_7cor t_u8_pb6 err_t_u8_pb6 t_u5_pb7 err_t_u5_pb7 t_pb7_pb6 err_t_pb7_pb6 t_th_pb8 err_t_th_pb8 if tipo==1, noheader compress
		
/* input parameters for plot*/

	display
	display
	display
	display as result ". INPUT OF AGE LIMITS FOR CONCORDIA PLOT"
	local lval "junk"
	capture confirm number `lval'
	while _rc!=0 {
  	          display as result". Lower Limit? " _cont
  	          display _request(_lval)
              capture confirm number `lval'
              } 
	local hval "junk"
	capture confirm number `hval'
	while _rc!=0 {
             display as result". Upper Limit? " _cont
             display _request(_hval)
             capture confirm number `hval'
             } 
	local tolab "junk"
	capture confirm number `tolab'
	while _rc!=0 {
             display as result". Labels to plot are multiple of? " _cont
             display _request(_interval)
             capture confirm number `interval'
             }       
	local witherrorbars "junk"             
	capture confirm string `witherrorbars'
	if _rc!=0 {
             display as result". Shall I plot error bars? (y/n)" _cont
             display _request(_witherrorbars)
             }
/*end input parameters for plot*/

/*preparo los graficos*/	

local j=`ndata'
quietly{
	gen t_plot=0
	gen caso=_n
	gen label=0
	replace tipo=0 if caso>`ndata' /*tipo es 1 si es una muestra, 0 si es auxiliar de proyeccion*/
	forvalues i=`lval'(`interval')`hval' {
          	 local j=`j'+1
          	 set obs `j'
          	 replace caso=`j' in `j'
          	 replace t_plot=`i' in `j'
          	 replace u8_pb6=1/(exp(t_plot*1.55125/10000)-1) in `j'
          	 replace pb7_pb6=(0.72*(exp(t_plot*9.8485/10000)-1))/(99.2743*(exp(t_plot*1.55125/10000)-1))  in `j'
		 replace id="to_plot" in `j'
		 replace tipo=0 in `j'
		 replace label=1 in `j'
             }
	}/*end quietly*/
/*calculo mximo y mnimo eje Y, para que las etiquetas no solapen*/

global age_pbpb=`lval'
	pbpb_calc $age_pbpb
	local min1=$pbpb_age
	local factorage=100
	if `min1'<0.1 local factorage=1000
	local min2=$pbpb_age*`factorage'
	local min3=floor(`min2')
	local ylabel_min=`min3'/`factorage'
global age_pbpb=`hval'
	pbpb_calc $age_pbpb
	local max1=$pbpb_age
	local factorage2=100	
	if `max1'<0.1 local factorage2=1000
	local max2=$pbpb_age*`factorage2'
	local max3=ceil(`max2')
	local ylabel_max=`max3'/`factorage2'
	

/* dos opciones de dibujar, con o sin error bars*/

	if "`witherrorbars'" =="y" {
	 twoway (line pb7_pb6 u8_pb6 if tipo==0, lcolor(blue)) (scatter pb7_pb6 u8_pb6 if tipo==0 & label==1, mcolor(blue) msymbol(square) mlabel(t_plot) msize(small) mlabposition(8)) (pcspike pb7_pb6 hi_u8_pb6 pb7_pb6 lo_u8_pb6 if tipo==1 & es ==1, lcolor(gs10) lwidth(thin)) (pcspike hi_pb7_pb6 u8_pb6 lo_pb7_pb6 u8_pb6 if tipo==1 & es ==1, lcolor(gs10) lwidth(thin)) (scatter pb7_pb6 u8_pb6 if tipo==1 & es ==1, mcolor(red) msymbol(circle) msize(small)), ytitle("207Pb/206Pb") yscale(range(`ylabel_min' `ylabel_max')) ylabel(, labsize(small) angle(horizontal) format(%9.3f)) ymtick(##5, nolabels) xtitle(238U/206Pb) xlabel(, labsize(small)) xmtick(##5, nolabels) title(`graph_title') legend(off) plotregion(lcolor(black))
	 	}
 		else {
 		twoway (line pb7_pb6 u8_pb6 if tipo==0, lcolor(blue)) (scatter pb7_pb6 u8_pb6 if tipo==0 & label==1, mcolor(blue) msymbol(square) mlabel(t_plot) msize(small) mlabposition(8)) (scatter pb7_pb6 u8_pb6 if tipo==1 & es ==1, mcolor(red) msymbol(circle)), ytitle("207Pb/206Pb") yscale(range(`ylabel_min' `ylabel_max')) ylabel(, labsize(small) angle(horizontal) format(%9.3f)) ymtick(##5, nolabels) xtitle(238U/206Pb) xlabel(, labsize(small)) xmtick(##5, nolabels) title(`graph_title') legend(off) plotregion(lcolor(black))
 		}
	quietly graph rename TW, replace
	quietly{
		drop if id=="to_plot"
		capture drop caso t_plot label 
		capture drop hi_pb7_pb6 lo_pb7_pb6 hi_u8_pb6 lo_u8_pb6 u8_pb6 u5_pb7 pb7_u5 th_pb8
		if  "`correction_204'"=="y"{
			replace pb7_pb6=pb7_pb6_uncor
			replace pb6_u8=pb6_u8_uncor
			replace pb8_th=pb8_th_uncor
			drop pb7_pb6_uncor pb6_u8_uncor pb8_th_uncor 
			}
		}/*del quietly*/
	
	display
	display as text ". Done Tera-Wasserburg"
	
end /*del ado*/





capture program define pbpb_calc /*calcula los valores pb7_pb6 correspondientes a los valores mximo y minimo definidos por el usuario para el grfico*/
	args age_pbpb
	local t=$age_pbpb
	global pbpb_age=(exp(9.84850*10^(-4)*`t')-1)/(137.88*(exp(1.55125*10^(-4)*`t')-1))
end	

capture program define t_pbpb
	args pbpb

	local t1=-99
		local t2=0
		local control=1
		while `control' > 0 {
			local t1=`t1' + 100 
			local pb7_pb6_0=(0.72*(exp(`t1'*9.8485/10000)-1))/(99.2743*(exp(`t1'*1.55125/10000)-1))			
			if `pb7_pb6_0' >= `pbpb' local control=0
			}
		local t2=`t1'- 100
		local t1=`t2'
		local control=1
		while `control' > 0 {
			local t1=`t1' + 10 
			local pb7_pb6_0=(0.72*(exp(`t1'*9.8485/10000)-1))/(99.2743*(exp(`t1'*1.55125/10000)-1))
			if `pb7_pb6_0' >=  `pbpb' local control=0
			}
		local t2=`t1'- 10
		local t1=`t2'
		local control=1
		while `control' > 0 {
			local t1=`t1' + 1 
			local pb7_pb6_0=(0.72*(exp(`t1'*9.8485/10000)-1))/(99.2743*(exp(`t1'*1.55125/10000)-1))
			if `pb7_pb6_0' >=  `pbpb' local control=0
			}
		local t2=`t1'- 1
		local t1=`t2'
		local control=1
		while `control' > 0 {
			local t1=`t1' + 0.2 
			local pb7_pb6_0=(0.72*(exp(`t1'*9.8485/10000)-1))/(99.2743*(exp(`t1'*1.55125/10000)-1))
			if `pb7_pb6_0' >=  `pbpb' local control=0
			}
		local t2=`t1'- 0.2

		global t_pbpb = (`t1'+`t2')/2
	end /*del t_pbpb*/	


capture program define cor207 	/*rutina para corregir la edad 238U/206Pb de Pb comun, por proyeccin en Tera-Wasserburg*/
	args t_upb pbpb
	local t2=`t_upb'
		local t1=`t2'+10
		local upb_c=1/(exp(`t2'*1.55125/10000)-1)
		local b= 0.835722 /*la ecuacion de la recta desde Pb comun a muestra*/
		local a=(`pbpb'-`b')/`upb_c'
		local control=1
		while `control' > 0 {
			local t1=`t1'-10 
			local pb7_pb6_c=(0.72*(exp(`t1'*9.8485/10000)-1))/(99.2743*(exp(`t1'*1.55125/10000)-1))
			local upb_c=1/(exp(`t1'*1.55125/10000)-1)
			local pb7_pb6_r=`upb_c'*`a'+`b'
			if `pb7_pb6_c' > `pb7_pb6_r' local control=0
			}
			local t2=`t1'+ 10
			local t1=`t2'
			local control=1	
		while `control' > 0 {
			local t1=`t1'-1
			local pb7_pb6_c=(0.72*(exp(`t1'*9.8485/10000)-1))/(99.2743*(exp(`t1'*1.55125/10000)-1))
			local upb_c=1/(exp(`t1'*1.55125/10000)-1)
			local pb7_pb6_r=`upb_c'*`a'+`b'
			if `pb7_pb6_c' > `pb7_pb6_r' local control=0
			}
  			local t2=`t1'+1
  			local t1=`t2'
			local control=1		
		while `control' > 0 {
			local t1=`t1'-0.1
			local pb7_pb6_c=(0.72*(exp(`t1'*9.8485/10000)-1))/(99.2743*(exp(`t1'*1.55125/10000)-1))
			local upb_c=1/(exp(`t1'*1.55125/10000)-1)
			local pb7_pb6_r=`upb_c'*`a'+`b'
			if `pb7_pb6_c' > `pb7_pb6_r' local control=0
			}
  			local t2=`t1'+0.05
		    global t_upbcor= `t2'
		   end/*del t_upb*/ 
		
				
capture program define sk 
		args p207 p208 p204

		local sk7=15.628-(9.735/137.88)*(exp(9.84850*10^(-4)*$t_pbpb)-1)
		local sk6=18.7-9.735*(exp(1.55125*10^(-4)*$t_pbpb)-1)
		local sk8=38.63-(36.837)*(exp(4.9475*10^(-5)*$t_pbpb)-1)
		local cden=1/`p204'-`sk6' 
		local cnum=`p207'*(1/`p204')-`sk7' 
		global pb207_4cor=`cnum'/`cden'	
		if $pb207_4cor<0 global pb207_4cor=0 
		global pb6_u8_4cor=`cden'*`p204'*pb6_u8[$i]
		local cnum2=`p208'*(1/`p204')-`sk8' 
		global pb208_4cor=`cnum2'/`cden'
		global pb8_th_4cor=`cnum2'*pb8_th[$i]/(`p208'*(1/`p204'))
		if pb4_pb6[$i] <= 0 | pb4_pb6[$i] ==.{	
			global pb207_4cor=pb7_pb6[$i]
			global pb208_4cor=pb8_pb6[$i]
			global pb6_u8_4cor=pb6_u8[$i]
			global pb8_th_4cor=pb8_th[$i]
			}
end


